"""

Records
=======

"""
# pylint:disable=R0903
from abc import ABCMeta


class Record(object):

    """All records are subclasses of the abstract class ``Record``"""
    __metaclass__ = ABCMeta

    _valid_attributes = None

    def __init__(self, **kwargs):
        valid_args = dict((k, kwargs.get(k)) for k in self._valid_attributes)
        self.__dict__.update(valid_args)

    def __setattr__(self, name, value):
        raise AttributeError("can't set attribute")


class PlaceRecord(Record):

    """All records with :py:attr:`names` subclass :py:class:`PlaceRecord`"""
    __metaclass__ = ABCMeta

    def __init__(self, locales=None, **kwargs):
        if locales is None:
            locales = ['en']
        if kwargs.get('names') is None:
            kwargs['names'] = {}
        object.__setattr__(self, '_locales', locales)
        super(PlaceRecord, self).__init__(**kwargs)

    @property
    def name(self):
        """Dict with locale codes as keys and localized name as value"""
        # pylint:disable=E1101
        return next((self.names.get(x) for x in self._locales if x in
                     self.names), None)


class City(PlaceRecord):

    """Contains data for the city record associated with an IP address

    This class contains the city-level data associated with an IP address.

    This record is returned by ``city`` and ``insights``.

    Attributes:

    .. attribute:: confidence

      A value from 0-100 indicating MaxMind's
      confidence that the city is correct. This attribute is only available
      from the Insights end point.

      :type: int

    .. attribute:: geoname_id

      The GeoName ID for the city.

      :type: int

    .. attribute:: name

      The name of the city based on the locales list passed to the
      constructor.

      :type: unicode

    .. attribute:: names

      A dictionary where the keys are locale codes
      and the values are names.

      :type: dict

    """
    _valid_attributes = set(['confidence', 'geoname_id', 'names'])


class Continent(PlaceRecord):

    """Contains data for the continent record associated with an IP address

    This class contains the continent-level data associated with an IP
    address.

    Attributes:


    .. attribute:: code

      A two character continent code like "NA" (North America)
      or "OC" (Oceania).

      :type: unicode

    .. attribute:: geoname_id

      The GeoName ID for the continent.

      :type: int

    .. attribute:: name

      Returns the name of the continent based on the locales list passed to
      the constructor.

      :type: unicode

    .. attribute:: names

      A dictionary where the keys are locale codes
      and the values are names.

      :type: dict

    """
    _valid_attributes = set(['code', 'geoname_id', 'names'])


class Country(PlaceRecord):

    """Contains data for the country record associated with an IP address

    This class contains the country-level data associated with an IP address.

    Attributes:


    .. attribute:: confidence

      A value from 0-100 indicating MaxMind's confidence that
      the country is correct. This attribute is only available from the
      Insights end point.

      :type: int

    .. attribute:: geoname_id

      The GeoName ID for the country.

      :type: int

    .. attribute:: iso_code

      The two-character `ISO 3166-1
      <http://en.wikipedia.org/wiki/ISO_3166-1>`_ alpha code for the
      country.

      :type: unicode

    .. attribute:: name

      The name of the country based on the locales list passed to the
      constructor.

      :type: unicode

    .. attribute:: names

      A dictionary where the keys are locale codes and the values
      are names.

      :type: dict

    """
    _valid_attributes = set(['confidence', 'geoname_id', 'iso_code', 'names'])


class RepresentedCountry(Country):

    """Contains data for the represented country associated with an IP address

    This class contains the country-level data associated with an IP address
    for the IP's represented country. The represented country is the country
    represented by something like a military base.

    Attributes:


    .. attribute:: confidence

      A value from 0-100 indicating MaxMind's confidence that
      the country is correct. This attribute is only available from the
      Insights end point.

      :type: int

    .. attribute:: geoname_id

      The GeoName ID for the country.

      :type: int

    .. attribute:: iso_code

      The two-character `ISO 3166-1
      <http://en.wikipedia.org/wiki/ISO_3166-1>`_ alpha code for the country.

      :type: unicode

    .. attribute:: name

      The name of the country based on the locales list passed to the
      constructor.

      :type: unicode

    .. attribute:: names

      A dictionary where the keys are locale codes and the values
      are names.

      :type: dict


    .. attribute:: type

      A string indicating the type of entity that is representing the
      country. Currently we only return ``military`` but this could expand to
      include other types in the future.

      :type: unicode

    """
    _valid_attributes = set(['confidence', 'geoname_id', 'iso_code',
                             'names', 'type'])


class Location(Record):

    """Contains data for the location record associated with an IP address

    This class contains the location data associated with an IP address.

    This record is returned by `city` and `insights`.

    Attributes:

    .. attribute:: accuracy_radius

      The radius in kilometers around the
      specified location where the IP address is likely to be. This attribute
      is only available from the Insights end point.

      :type: int

    .. attribute:: latitude

      The latitude of the location as a floating point number.

      :type: float


    .. attribute:: longitude

      The longitude of the location as a
      floating point number.

      :type: float

    .. attribute:: metro_code

      The metro code of the location if the
      location is in the US. MaxMind returns the same metro codes as the
      `Google AdWords API
      <https://developers.google.com/adwords/api/docs/appendix/cities-DMAregions>`_.

      :type: int

    .. attribute:: time_zone

      The time zone associated with location, as
      specified by the `IANA Time Zone Database
      <http://www.iana.org/time-zones>`_, e.g., "America/New_York".

      :type: unicode

    """
    _valid_attributes = set(['accuracy_radius', 'latitude', 'longitude',
                             'metro_code', 'postal_code', 'postal_confidence',
                             'time_zone'])


class MaxMind(Record):

    """Contains data related to your MaxMind account

    Attributes:

    .. attribute:: queries_remaining

      The number of remaining queries you have
      for the end point you are calling.

      :type: int

    """
    _valid_attributes = set(['queries_remaining'])


class Postal(Record):

    """Contains data for the postal record associated with an IP address

    This class contains the postal data associated with an IP address.

    This attribute is returned by ``city`` and ``insights``.

    Attributes:

    .. attribute:: code

      The postal code of the location. Postal
      codes are not available for all countries. In some countries, this will
      only contain part of the postal code.

      :type: unicode

    .. attribute:: confidence

      A value from 0-100 indicating
      MaxMind's confidence that the postal code is correct. This attribute is
      only available from the Insights end point.

      :type: int

    """
    _valid_attributes = set(['code', 'confidence'])


class Subdivision(PlaceRecord):

    """Contains data for the subdivisions associated with an IP address

    This class contains the subdivision data associated with an IP address.

    This attribute is returned by ``city`` and ``insights``.

    Attributes:

    .. attribute:: confidence

      This is a value from 0-100 indicating MaxMind's
      confidence that the subdivision is correct. This attribute is only
      available from the Insights end point.

      :type: int

    .. attribute:: geoname_id

      This is a GeoName ID for the subdivision.

      :type: int

    .. attribute:: iso_code

      This is a string up to three characters long
      contain the subdivision portion of the `ISO 3166-2 code
      <http://en.wikipedia.org/wiki/ISO_3166-2>`_.

      :type: unicode

    .. attribute:: name

      The name of the subdivision based on the locales list passed to the
      constructor.

      :type: unicode

    .. attribute:: names

      A dictionary where the keys are locale codes and the
      values are names

      :type: dict

    """
    _valid_attributes = set(['confidence', 'geoname_id', 'iso_code', 'names'])


class Subdivisions(tuple):

    """A tuple-like collection of subdivisions associated with an IP address

    This class contains the subdivisions of the country associated with the
    IP address from largest to smallest.

    For instance, the response for Oxford in the United Kingdom would have
    England as the first element and Oxfordshire as the second element.

    This attribute is returned by ``city`` and ``insights``.
    """
    # pylint:disable=W0142
    def __new__(cls, locales, *subdivisions):
        subdivisions = [Subdivision(locales, **x) for x in subdivisions]
        obj = super(cls, Subdivisions).__new__(cls, subdivisions)
        return obj

    def __init__(self, locales, *subdivisions):  # pylint:disable=W0613
        self._locales = locales
        super(Subdivisions, self).__init__()

    @property
    def most_specific(self):
        """The most specific (smallest) subdivision available.

          If there are no :py:class:`Subdivision` objects for the response,
          this returns an empty :py:class:`Subdivision`.

          :type: :py:class:`Subdivision`
        """
        try:
            return self[-1]
        except IndexError:
            return Subdivision(self._locales)


class Traits(Record):

    """ Contains data for the traits record associated with an IP address

    This class contains the traits data associated with an IP address.

    This class has the following attributes:


    .. attribute:: autonomous_system_number

      The `autonomous system
      number <http://en.wikipedia.org/wiki/Autonomous_system_(Internet)>`_
      associated with the IP address. This attribute is only available from
      the City and Insights web service end points.

      :type: int

    .. attribute:: autonomous_system_organization

      The organization associated with the registered `autonomous system
      number <http://en.wikipedia.org/wiki/Autonomous_system_(Internet)>`_ for
      the IP address. This attribute is only available from the City and
      Insights web service end points.

      :type: unicode

    .. attribute:: domain

      The second level domain associated with the
      IP address. This will be something like "example.com" or
      "example.co.uk", not "foo.example.com". This attribute is only available
      from the City and Insights web service end points.

      :type: unicode

    .. attribute:: ip_address

      The IP address that the data in the model
      is for. If you performed a "me" lookup against the web service, this
      will be the externally routable IP address for the system the code is
      running on. If the system is behind a NAT, this may differ from the IP
      address locally assigned to it.

      :type: unicode

    .. attribute:: is_anonymous_proxy

      This is true if the IP is an anonymous
      proxy. See http://dev.maxmind.com/faq/geoip#anonproxy for further
      details.

      :type: bool

    .. attribute:: is_satellite_provider

      This is true if the IP address is from a satellite provider that
      provides service to multiple countries.

      :type: bool

    .. attribute:: isp

      The name of the ISP associated with the IP address. This attribute is
      only available from the City and Insights web service end points.

      :type: unicode

    .. attribute:: organization

      The name of the organization associated with the IP address. This
      attribute is only available from the City and Insights web service end
      points.

      :type: unicode

    .. attribute:: user_type

      The user type associated with the IP
      address. This can be one of the following values:

      * business
      * cafe
      * cellular
      * college
      * content_delivery_network
      * dialup
      * government
      * hosting
      * library
      * military
      * residential
      * router
      * school
      * search_engine_spider
      * traveler

      This attribute is only available from the Insights end point.

      :type: unicode

"""
    _valid_attributes = set(['autonomous_system_number',
                             'autonomous_system_organization',
                             'domain',
                             'is_anonymous_proxy',
                             'is_satellite_provider',
                             'isp',
                             'ip_address',
                             'organization',
                             'user_type'])

    def __init__(self, **kwargs):
        for k in ['is_anonymous_proxy', 'is_satellite_provider']:
            kwargs[k] = bool(kwargs.get(k, False))
        super(Traits, self).__init__(**kwargs)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import sys
sys.path.append('..')

import geoip2.database

if sys.version_info[:2] == (2, 6):
    import unittest2 as unittest
else:
    import unittest

if sys.version_info[0] == 2:
    unittest.TestCase.assertRaisesRegex = unittest.TestCase.assertRaisesRegexp


class TestReader(unittest.TestCase):

    def test_default_locale(self):
        reader = geoip2.database.Reader(
            'tests/data/test-data/GeoIP2-City-Test.mmdb')
        record = reader.city('81.2.69.160')

        self.assertEqual(record.country.name, 'United Kingdom')
        reader.close()

    def test_language_list(self):
        reader = geoip2.database.Reader(
            'tests/data/test-data/GeoIP2-Country-Test.mmdb',
            ['xx', 'ru', 'pt-BR', 'es', 'en'])
        record = reader.country('81.2.69.160')

        self.assertEqual(record.country.name, 'Великобритания')
        reader.close()

    def test_has_ip_address(self):
        reader = geoip2.database.Reader(
            'tests/data/test-data/GeoIP2-Country-Test.mmdb')
        record = reader.country('81.2.69.160')
        self.assertEqual(record.traits.ip_address, '81.2.69.160')
        reader.close()

    def test_unknown_address(self):
        reader = geoip2.database.Reader(
            'tests/data/test-data/GeoIP2-City-Test.mmdb')
        with self.assertRaisesRegex(geoip2.errors.AddressNotFoundError,
                                    'The address 10.10.10.10 is not in the '
                                    'database.'):
            reader.city('10.10.10.10')
        reader.close()

    def test_wrong_database(self):
        reader = geoip2.database.Reader(
            'tests/data/test-data/GeoIP2-City-Test.mmdb')
        with self.assertRaisesRegex(TypeError,
                                    'The country method cannot be used with '
                                    'the GeoIP2-City database'):
            reader.country('1.1.1.1')
        reader.close()

    def test_invalid_address(self):
        reader = geoip2.database.Reader(
            'tests/data/test-data/GeoIP2-City-Test.mmdb')
        with self.assertRaisesRegex(ValueError,
                                    "u?'invalid' does not appear to be an "
                                    "IPv4 or IPv6 address"):
            reader.city('invalid')
        reader.close()

    def test_connection_type(self):
        reader = geoip2.database.Reader(
            'tests/data/test-data/GeoIP2-Connection-Type-Test.mmdb')
        ip_address = '1.0.1.0'

        record = reader.connection_type(ip_address)
        self.assertEqual(record.connection_type, 'Cable/DSL')
        self.assertEqual(record.ip_address, ip_address)
        reader.close()

    def test_domain(self):
        reader = geoip2.database.Reader(
            'tests/data/test-data/GeoIP2-Domain-Test.mmdb')

        ip_address = '1.2.0.0'
        record = reader.domain(ip_address)
        self.assertEqual(record.domain, 'maxmind.com')
        self.assertEqual(record.ip_address, ip_address)

        reader.close()

    def test_isp(self):
        reader = geoip2.database.Reader(
            'tests/data/test-data/GeoIP2-ISP-Test.mmdb')

        ip_address = '1.128.0.0'
        record = reader.isp(ip_address)
        self.assertEqual(record.autonomous_system_number, 1221)
        self.assertEqual(record.autonomous_system_organization,
                         'Telstra Pty Ltd')
        self.assertEqual(record.isp, 'Telstra Internet')
        self.assertEqual(record.organization, 'Telstra Internet')
        self.assertEqual(record.ip_address, ip_address)

        reader.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import sys
sys.path.append('..')

import geoip2.models

if sys.version_info[:2] == (2, 6):
    import unittest2 as unittest
else:
    import unittest

if sys.version_info[0] == 2:
    unittest.TestCase.assertRaisesRegex = unittest.TestCase.assertRaisesRegexp


class TestModels(unittest.TestCase):

    def test_insights_full(self):
        raw = {
            'city': {
                'confidence': 76,
                'geoname_id': 9876,
                'names': {'en': 'Minneapolis'},
            },
            'continent': {
                'code': 'NA',
                'geoname_id': 42,
                'names': {'en': 'North America'},
            },
            'country': {
                'confidence': 99,
                'geoname_id': 1,
                'iso_code': 'US',
                'names': {'en': 'United States of America'},
            },
            'location': {
                'accuracy_radius': 1500,
                'latitude': 44.98,
                'longitude': 93.2636,
                'metro_code': 765,
                'time_zone': 'America/Chicago',
            },
            'postal': {
                'code': '55401',
                'confidence': 33,
            },
            'subdivisions': [{
                'confidence': 88,
                'geoname_id': 574635,
                'iso_code': 'MN',
                'names': {'en': 'Minnesota'},
            },
                {
                    'geoname_id': 123,
                    'iso_code': 'HP',
                    'names': {'en': 'Hennepin'},
                }
            ],
            'registered_country': {
                'geoname_id': 2,
                'iso_code': 'CA',
                'names': {'en': 'Canada'},
            },
            'represented_country': {
                'geoname_id': 3,
                'iso_code': 'GB',
                'names': {'en': 'United Kingdom'},
                'type': 'military',
            },
            'traits': {
                'autonomous_system_number': 1234,
                'autonomous_system_organization': 'AS Organization',
                'domain': 'example.com',
                'ip_address': '1.2.3.4',
                'is_anonymous_proxy': 0,
                'is_us_military': 1,
                'isp': 'Comcast',
                'network_speed': 'cable/DSL',
                'organization': 'Blorg',
                'user_type': 'college',
            },
        }

        model = geoip2.models.Insights(raw)
        self.assertEqual(type(model), geoip2.models.Insights,
                         'geoip2.models.Insights object')
        self.assertEqual(type(model.city), geoip2.records.City,
                         'geoip2.records.City object')
        self.assertEqual(type(model.continent), geoip2.records.Continent,
                         'geoip2.records.Continent object')
        self.assertEqual(type(model.country), geoip2.records.Country,
                         'geoip2.records.Country object')
        self.assertEqual(type(model.registered_country),
                         geoip2.records.Country,
                         'geoip2.records.Country object')
        self.assertEqual(type(model.represented_country),
                         geoip2.records.RepresentedCountry,
                         'geoip2.records.RepresentedCountry object')
        self.assertEqual(type(model.location), geoip2.records.Location,
                         'geoip2.records.Location object')
        self.assertEqual(type(model.subdivisions[0]),
                         geoip2.records.Subdivision,
                         'geoip2.records.Subdivision object')
        self.assertEqual(type(model.traits), geoip2.records.Traits,
                         'geoip2.records.Traits object')
        self.assertEqual(model.raw, raw, 'raw method returns raw input')
        self.assertEqual(model.subdivisions[0].iso_code, 'MN',
                         'div 1 has correct iso_code')
        self.assertEqual(model.subdivisions[0].confidence, 88,
                         'div 1 has correct confidence')
        self.assertEqual(model.subdivisions[0].geoname_id, 574635,
                         'div 1 has correct geoname_id')
        self.assertEqual(model.subdivisions[0].names,
                         {'en': 'Minnesota'}, 'div 1 names are correct')
        self.assertEqual(model.subdivisions[1].name, 'Hennepin',
                         'div 2 has correct name')
        self.assertEqual(model.subdivisions.most_specific.iso_code, 'HP',
                         'subdivisions.most_specific returns HP')
        self.assertEqual(model.represented_country.name,
                         'United Kingdom',
                         'represented_country name is correct')
        self.assertEqual(model.represented_country.type,
                         'military',
                         'represented_country type is correct')
        self.assertEqual(model.location.latitude, 44.98,
                         'correct latitude')
        self.assertEqual(model.location.longitude, 93.2636,
                         'correct longitude')
        self.assertEqual(model.location.metro_code, 765,
                         'correct metro_code')

    def test_insights_min(self):
        model = geoip2.models.Insights({'traits': {'ip_address': '5.6.7.8'}})
        self.assertEqual(type(model), geoip2.models.Insights,
                         'geoip2.models.Insights object')
        self.assertEqual(type(model.city), geoip2.records.City,
                         'geoip2.records.City object')
        self.assertEqual(type(model.continent), geoip2.records.Continent,
                         'geoip2.records.Continent object')
        self.assertEqual(type(model.country), geoip2.records.Country,
                         'geoip2.records.Country object')
        self.assertEqual(type(model.registered_country),
                         geoip2.records.Country,
                         'geoip2.records.Country object')
        self.assertEqual(type(model.location), geoip2.records.Location,
                         'geoip2.records.Location object')
        self.assertEqual(type(model.traits), geoip2.records.Traits,
                         'geoip2.records.Traits object')
        self.assertEqual(type(model.subdivisions.most_specific),
                         geoip2.records.Subdivision,
                         'geoip2.records.Subdivision object returned even'
                         'when none are available.')
        self.assertEqual(model.subdivisions.most_specific.names, {},
                         'Empty names hash returned')

    def test_city_full(self):
        raw = {
            'continent': {
                'code': 'NA',
                'geoname_id': 42,
                'names': {'en': 'North America'},
            },
            'country': {
                'geoname_id': 1,
                'iso_code': 'US',
                'names': {'en': 'United States of America'},
            },
            'registered_country': {
                'geoname_id': 2,
                'iso_code': 'CA',
                'names': {'en': 'Canada'},
            },
            'traits': {
                'ip_address': '1.2.3.4',
                'is_satellite_provider': True,
            },
        }
        model = geoip2.models.City(raw)
        self.assertEqual(type(model), geoip2.models.City,
                         'geoip2.models.City object')
        self.assertEqual(type(model.city), geoip2.records.City,
                         'geoip2.records.City object')
        self.assertEqual(type(model.continent), geoip2.records.Continent,
                         'geoip2.records.Continent object')
        self.assertEqual(type(model.country), geoip2.records.Country,
                         'geoip2.records.Country object')
        self.assertEqual(type(model.registered_country),
                         geoip2.records.Country,
                         'geoip2.records.Country object')
        self.assertEqual(type(model.location), geoip2.records.Location,
                         'geoip2.records.Location object')
        self.assertEqual(type(model.traits), geoip2.records.Traits,
                         'geoip2.records.Traits object')
        self.assertEqual(model.raw, raw, 'raw method returns raw input')
        self.assertEqual(model.continent.geoname_id, 42,
                         'continent geoname_id is 42')
        self.assertEqual(model.continent.code, 'NA',
                         'continent code is NA')
        self.assertEqual(model.continent.names, {'en': 'North America'},
                         'continent names is correct')
        self.assertEqual(model.continent.name, 'North America',
                         'continent name is correct')
        self.assertEqual(model.country.geoname_id, 1,
                         'country geoname_id is 1')
        self.assertEqual(model.country.iso_code, 'US',
                         'country iso_code is US')
        self.assertEqual(model.country.names,
                         {'en': 'United States of America'},
                         'country names is correct')
        self.assertEqual(model.country.name, 'United States of America',
                         'country name is correct')
        self.assertEqual(model.country.confidence, None,
                         'country confidence is None')
        self.assertEqual(model.registered_country.iso_code, 'CA',
                         'registered_country iso_code is CA')
        self.assertEqual(model.registered_country.names,
                         {'en': 'Canada'},
                         'registered_country names is correct')
        self.assertEqual(model.registered_country.name,
                         'Canada',
                         'registered_country name is correct')
        self.assertEqual(model.traits.is_anonymous_proxy, False,
                         'traits is_anonymous_proxy returns False by default')
        self.assertEqual(model.traits.is_satellite_provider, True,
                         'traits is_setellite_provider is True')
        self.assertEqual(model.raw, raw, 'raw method produces raw output')

    def test_unknown_keys(self):
        model = geoip2.models.City({'traits': {'ip_address': '1.2.3.4',
                                               'invalid': 'blah'},
                                    'unk_base': {'blah': 1}})
        with self.assertRaises(AttributeError):
            model.unk_base
        with self.assertRaises(AttributeError):
            model.traits.invalid
        self.assertEqual(model.traits.ip_address, '1.2.3.4', 'correct ip')


class TestNamess(unittest.TestCase):

    raw = {
        'continent': {
            'code': 'NA',
            'geoname_id': 42,
            'names': {
                'de': 'Nordamerika',
                'en': 'North America',
                'es': 'América del Norte',
                'ja': '北アメリカ',
                'pt-BR': 'América do Norte',
                'ru': 'Северная Америка',
                'zh-CN': '北美洲',
            },
        },
        'country': {
            'geoname_id': 1,
            'iso_code': 'US',
            'names': {
                'en': 'United States of America',
                'fr': 'États-Unis',
                'zh-CN': '美国',
            },
        },
        'traits': {
            'ip_address': '1.2.3.4',
        },
    }

    def test_names(self):
        model = geoip2.models.Country(self.raw, locales=['sq', 'ar'])
        self.assertEqual(model.continent.names,
                         self.raw['continent']['names'],
                         'Correct names dict for continent')
        self.assertEqual(model.country.names,
                         self.raw['country']['names'],
                         'Correct names dict for country')

    def test_three_locales(self):
        model = geoip2.models.Country(self.raw,
                                      locales=['fr', 'zh-CN', 'en'])
        self.assertEqual(model.continent.name, '北美洲',
                         'continent name is in Chinese (no French available)')
        self.assertEqual(model.country.name, 'États-Unis',
                         'country name is in French')

    def test_two_locales(self):
        model = geoip2.models.Country(self.raw, locales=['ak', 'fr'])
        self.assertEqual(model.continent.name, None,
                         'continent name is undef (no Akan or French '
                         'available)')
        self.assertEqual(model.country.name, 'États-Unis',
                         'country name is in French')

    def test_unknown_locale(self):
        model = geoip2.models.Country(self.raw, locales=['aa'])
        self.assertEqual(model.continent.name, None,
                         'continent name is undef (no Afar available)')
        self.assertEqual(model.country.name, None,
                         'country name is in None (no Afar available)')

    def test_german(self):
        model = geoip2.models.Country(self.raw, locales=['de'])
        self.assertEqual(model.continent.name, 'Nordamerika',
                         'Correct german name for continent')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

import codecs
import os
import sys

import geoip2

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

packages = ['geoip2']

requirements = [i.strip() for i in open("requirements.txt").readlines()]

setup(
    name='geoip2',
    version=geoip2.__version__,
    description='MaxMind GeoIP2 API',
    long_description=codecs.open('README.rst', 'r', 'utf-8').read(),
    author='Gregory Oschwald',
    author_email='goschwald@maxmind.com',
    url='http://www.maxmind.com/',
    bugtrack_url='https://github.com/maxmind/GeoIP2-python/issues',
    packages=['geoip2'],
    package_data={'': ['LICENSE']},
    package_dir={'geoip2': 'geoip2'},
    include_package_data=True,
    install_requires=requirements,
    extras_require={
        ':python_version=="2.6" or python_version=="2.7"': ['ipaddr']},
    tests_require=['httpretty>=0.6.1'],
    test_suite="tests",
    license=geoip2.__license__,
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python',
        'Topic :: Internet :: Proxy Servers',
        'Topic :: Internet',
    ),
)



"""
======================
GeoIP2 Database Reader
======================

"""
import inspect

import geoip2
import geoip2.models
import geoip2.errors
import maxminddb


class Reader(object):

    """Creates a new GeoIP2 database Reader object.

    Instances of this class provide a reader for the GeoIP2 database format.
    IP addresses can be looked up using the ``country`` and ``city`` methods.

     Usage
     -----

    The basic API for this class is the same for every database. First, you
    create a reader object, specifying a file name. You then call the method
    corresponding to the specific database, passing it the IP address you want
    to look up.

    If the request succeeds, the method call will return a model class for the
    method you called. This model in turn contains multiple record classes,
    each of which represents part of the data returned by the database. If the
    database does not contain the requested information, the attributes on the
    record class will have a ``None`` value.

    If the address is not in the database, an
    ``geoip2.errors.AddressNotFoundError`` exception will be thrown. If the
    database is corrupt or invalid, a ``maxminddb.InvalidDatabaseError`` will
    be thrown.

"""

    def __init__(self, filename, locales=None):
        if locales is None:
            locales = ['en']
        self._db_reader = maxminddb.Reader(filename)
        self._locales = locales

    def country(self, ip_address):
        """Get the Country object for the IP address

        :param ip_address: IPv4 or IPv6 address as a string.

        :returns: :py:class:`geoip2.models.Country` object

        """

        return self._model_for(geoip2.models.Country, 'Country', ip_address)

    def city(self, ip_address):
        """Get the City object for the IP address

        :param ip_address: IPv4 or IPv6 address as a string.

        :returns: :py:class:`geoip2.models.City` object

        """
        return self._model_for(geoip2.models.City, 'City', ip_address)

    def connection_type(self, ip_address):
        """Get the ConnectionType object for the IP address

        :param ip_address: IPv4 or IPv6 address as a string.

        :returns: :py:class:`geoip2.models.ConnectionType` object

        """
        return self._flat_model_for(geoip2.models.ConnectionType,
                                    'GeoIP2-Connection-Type',
                                    ip_address)

    def domain(self, ip_address):
        """Get the Domain object for the IP address

        :param ip_address: IPv4 or IPv6 address as a string.

        :returns: :py:class:`geoip2.models.Domain` object

        """
        return self._flat_model_for(geoip2.models.Domain,
                                    'GeoIP2-Domain',
                                    ip_address)

    def isp(self, ip_address):
        """Get the ISP object for the IP address

        :param ip_address: IPv4 or IPv6 address as a string.

        :returns: :py:class:`geoip2.models.ISP` object

        """
        return self._flat_model_for(geoip2.models.ISP,
                                    'GeoIP2-ISP',
                                    ip_address)

    def _get(self, database_type, ip_address):
        if not database_type in self.metadata().database_type:
            caller = inspect.stack()[2][3]
            raise TypeError("The %s method cannot be used with the "
                            "%s database" %
                            (caller, self.metadata().database_type))
        record = self._db_reader.get(ip_address)
        if record is None:
            raise geoip2.errors.AddressNotFoundError(
                "The address %s is not in the database." % ip_address)
        return record

    def _model_for(self, model_class, types, ip_address):
        record = self._get(types, ip_address)
        record.setdefault('traits', {})['ip_address'] = ip_address
        return model_class(record, locales=self._locales)

    def _flat_model_for(self, model_class, types, ip_address):
        record = self._get(types, ip_address)
        record['ip_address'] = ip_address
        return model_class(record)

    def metadata(self):
        """The metadata for the open database

        :returns: :py:class:`maxminddb.reader.Metadata` object
        """
        return self._db_reader.metadata()

    def close(self):
        """Closes the GeoIP2 database"""

        self._db_reader.close()
